@@ -40,6 +40,7 @@ from cinderclient import api_versions
 from cinderclient import exceptions
 import cinderclient.extension
 from cinderclient._i18n import _
+from cinderclient._i18n import _LW
 from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import strutils
@@ -72,6 +73,37 @@ for svc in ('volume', 'volumev2', 'volumev3'):
     discover.add_catalog_discover_hack(svc, re.compile('/v[12]/\w+/?$'), '/')
 
 
+def get_server_version(url):
+    """Queries the server via the naked endpoint and gets version info.
+
+    :param url: url of the cinder endpoint
+    :returns: APIVersion object for min and max version supported by
+              the server
+    """
+
+    logger = logging.getLogger(__name__)
+    try:
+        scheme, netloc, path, query, frag = urlparse.urlsplit(url)
+        response = requests.get(scheme + '://' + netloc)
+        data = json.loads(response.text)
+        versions = data['versions']
+        for version in versions:
+            if '3.' in version['version']:
+                return (api_versions.APIVersion(version['min_version']),
+                        api_versions.APIVersion(version['version']))
+    except exceptions.ClientException as e:
+        logger.warning(_LW("Error in server version query:%s\n"
+                 "Returning APIVersion 2.0"), six.text_type(e.message))
+        return api_versions.APIVersion("2.0"), api_versions.APIVersion("2.0")
+
+
+def get_highest_client_server_version(url):
+    min_server, max_server = get_server_version(url)
+    max_server_version = api_versions.APIVersion.get_string(max_server)
+
+    return min(float(max_server_version), float(api_versions.MAX_VERSION))
+
+
 def get_volume_api_from_url(url):
     scheme, netloc, path, query, frag = urlparse.urlsplit(url)
     components = path.split("/")
@@ -115,10 +147,6 @@ class SessionClient(adapter.LegacyJsonAdapter):
                                                         raise_exc=False,
                                                         **kwargs)
 
-        # if service name is None then use service_type for logging
-        service = self.service_name or self.service_type
-        _log_request_id(self._logger, resp, service)
-
         if raise_exc and resp.status_code >= 400:
             raise exceptions.from_response(resp, body)
 
@@ -182,7 +210,7 @@ class SessionClient(adapter.LegacyJsonAdapter):
                              'auth plugin.')
 
     def _cs_request_base_url(self, url, method, **kwargs):
-        base_url = self._get_base_url(**kwargs)
+        base_url = self._get_base_url()
         return self._cs_request(
             base_url + url,
             method,
@@ -205,7 +233,8 @@ class HTTPClient(object):
                  bypass_url=None, retries=None,
                  http_log_debug=False, cacert=None,
                  auth_system='keystone', auth_plugin=None, api_version=None,
-                 logger=None):
+                 logger=None, user_domain_name='Default',
+                 project_domain_name='Default'):
         self.user = user
         self.password = password
         self.projectid = projectid
@@ -236,6 +265,8 @@ class HTTPClient(object):
         self.proxy_token = proxy_token
         self.proxy_tenant_id = proxy_tenant_id
         self.timeout = timeout
+        self.user_domain_name = user_domain_name
+        self.project_domain_name = project_domain_name
 
         if insecure:
             self.verify_cert = False
@@ -419,8 +450,8 @@ class HTTPClient(object):
         We may get redirected to another site, fail or actually get
         back a service catalog with a token and our endpoints.
         """
-
-        if resp.status_code == 200:  # content must always present
+        # content must always present
+        if resp.status_code == 200 or resp.status_code == 201:
             try:
                 self.auth_url = url
                 self.auth_ref = access.create(resp=resp, body=body)
@@ -497,10 +528,10 @@ class HTTPClient(object):
                                          path, query, frag))
 
         auth_url = self.auth_url
-        if self.version == "v2.0":
+        if self.version == "v2.0" or self.version == "v3":
             while auth_url:
                 if not self.auth_system or self.auth_system == 'keystone':
-                    auth_url = self._v2_auth(auth_url)
+                    auth_url = self._v2_or_v3_auth(auth_url)
                 else:
                     auth_url = self._plugin_auth(auth_url)
 
@@ -526,7 +557,7 @@ class HTTPClient(object):
             except exceptions.AuthorizationFailure:
                 if auth_url.find('v2.0') < 0:
                     auth_url = auth_url + '/v2.0'
-                self._v2_auth(auth_url)
+                self._v2_or_v3_auth(auth_url)
 
         if self.bypass_url:
             self.set_management_url(self.bypass_url)
@@ -559,23 +590,43 @@ class HTTPClient(object):
     def _plugin_auth(self, auth_url):
         return self.auth_plugin.authenticate(self, auth_url)
 
-    def _v2_auth(self, url):
+    def _v2_or_v3_auth(self, url):
         """Authenticate against a v2.0 auth service."""
-        body = {"auth": {
-            "passwordCredentials": {"username": self.user,
-                                    "password": self.password}}}
+        if self.version == "v3":
+            body = {
+                "auth": {
+                    "identity": {
+                        "methods": ["password"],
+                        "password": {"user": {
+                            "domain": {"name": self.user_domain_name},
+                            "name": self.user,
+                            "password": self.password}}},
+                }
+            }
+            scope = {"project": {"domain": {"name": self.project_domain_name}}}
+            if self.projectid:
+                scope['project']['name'] = self.projectid
+            elif self.tenant_id:
+                scope['project']['id'] = self.tenant_id
 
-        if self.projectid:
-            body['auth']['tenantName'] = self.projectid
-        elif self.tenant_id:
-            body['auth']['tenantId'] = self.tenant_id
+            body["auth"]["scope"] = scope
+        else:
+            body = {"auth": {
+                "passwordCredentials": {"username": self.user,
+                                        "password": self.password}}}
 
+            if self.projectid:
+                body['auth']['tenantName'] = self.projectid
+            elif self.tenant_id:
+                body['auth']['tenantId'] = self.tenant_id
         self._authenticate(url, body)
 
     def _authenticate(self, url, body):
         """Authenticate and extract the service catalog."""
-        token_url = url + "/tokens"
-
+        if self.version == 'v3':
+            token_url = url + "/auth/tokens"
+        else:
+            token_url = url + "/tokens"
         # Make sure we follow redirects when trying to reach Keystone
         resp, body = self.request(
             token_url,
@@ -719,7 +770,7 @@ def Client(version, *args, **kwargs):
 
         >>> from cinderclient import client
         >>> cinder = client.Client(VERSION, USERNAME, PASSWORD,
-        ...                      PROJECT_ID, AUTH_URL)
+        ...                      PROJECT_NAME, AUTH_URL)
 
     Here ``VERSION`` can be a string or
     ``cinderclient.api_versions.APIVersion`` obj. If you prefer string value,
