@@ -218,6 +218,14 @@ class ShellTest(utils.TestCase):
         mock_print.assert_called_once_with(mock.ANY, key_list,
             exclude_unavailable=True, sortby_index=0)
 
+    @mock.patch("cinderclient.utils.print_list")
+    def test_list_duplicate_fields(self, mock_print):
+        self.run_command('list --field Status,id,Size,status')
+        self.assert_called('GET', '/volumes/detail')
+        key_list = ['ID', 'Status', 'Size']
+        mock_print.assert_called_once_with(mock.ANY, key_list,
+            exclude_unavailable=True, sortby_index=0)
+
     @mock.patch("cinderclient.utils.print_list")
     def test_list_field_with_tenant(self, mock_print):
         self.run_command('list --field Status,Name,Size,Bootable '
@@ -449,14 +457,14 @@ class ShellTest(utils.TestCase):
                           'restore_vol')
 
     @ddt.data('backup_name', '1234')
-    @mock.patch('cinderclient.v3.shell._find_backup')
+    @mock.patch('cinderclient.shell_utils.find_backup')
     @mock.patch('cinderclient.utils.print_dict')
     @mock.patch('cinderclient.utils.find_volume')
     def test_do_backup_restore_with_name(self,
-                               value,
-                               mock_find_volume,
-                               mock_print_dict,
-                               mock_find_backup):
+                                         value,
+                                         mock_find_volume,
+                                         mock_print_dict,
+                                         mock_find_backup):
         backup_id = '1234'
         volume_id = '5678'
         name = None
@@ -471,11 +479,11 @@ class ShellTest(utils.TestCase):
                                'restore') as mocked_restore:
             mock_find_volume.return_value = volumes.Volume(self,
                                                            {'id': volume_id},
-                                                           loaded = True)
+                                                           loaded=True)
             mock_find_backup.return_value = volume_backups.VolumeBackup(
                 self,
                 {'id': backup_id},
-                loaded = True)
+                loaded=True)
             test_shell.do_backup_restore(self.cs, args)
             mock_find_backup.assert_called_once_with(
                 self.cs,
@@ -505,6 +513,10 @@ class ShellTest(utils.TestCase):
         self.assert_called('GET', '/snapshots/detail?'
                            'status=available&volume_id=1234')
 
+    def test_snapshot_list_filter_name(self):
+        self.run_command('snapshot-list --name abc')
+        self.assert_called('GET', '/snapshots/detail?name=abc')
+
     @mock.patch("cinderclient.utils.print_list")
     def test_snapshot_list_sort(self, mock_print_list):
         self.run_command('snapshot-list --sort id')
@@ -734,6 +746,10 @@ class ShellTest(utils.TestCase):
                           '--description=test_type-1-desc '
                           '--is-public=invalid_bool 1')
 
+    def test_type_update_without_args(self):
+        self.assertRaises(exceptions.CommandError, self.run_command,
+                          'type-update 1')
+
     def test_type_access_list(self):
         self.run_command('type-access-list --volume-type 3')
         self.assert_called('GET', '/types/3/os-volume-type-access')
