@@ -17,11 +17,13 @@
 from __future__ import print_function
 
 import argparse
+import collections
 import os
 
 from oslo_utils import strutils
 import six
 
+import cinderclient
 from cinderclient import api_versions
 from cinderclient import base
 from cinderclient import exceptions
@@ -30,9 +32,12 @@ from cinderclient import utils
 
 from cinderclient.v2.shell import *  # flake8: noqa
 
-utils.retype_method('volumev2', 'volumev3', globals())
-
 
+@utils.arg('--group_id',
+           metavar='<group_id>',
+           default=None,
+           help='Filters results by a group_id. Default=None.',
+           start_version='3.10')
 @utils.arg('--all-tenants',
            dest='all_tenants',
            metavar='<0|1>',
@@ -72,15 +77,16 @@ utils.retype_method('volumev2', 'volumev3', globals())
            nargs='*',
            metavar='<key=value>',
            default=None,
-           help='Filters results by a metadata key and value pair. Require '
-                'volume api version >=3.4. Default=None.')
+           help='Filters results by a metadata key and value pair. '
+                'Default=None.')
 @utils.arg('--image_metadata',
            type=str,
            nargs='*',
            metavar='<key=value>',
            default=None,
-           help='Filters results by a image metadata key and value pair. '
-                'Default=None.')
+           start_version='3.4',
+           help='Filters results by a image metadata key and value pair. Require '
+                'volume api version >=3.4. Default=None.')
 @utils.arg('--marker',
            metavar='<marker>',
            default=None,
@@ -119,7 +125,6 @@ utils.retype_method('volumev2', 'volumev3', globals())
            nargs='?',
            metavar='<tenant>',
            help='Display information from single tenant (Admin only).')
-@utils.service_type('volumev3')
 def do_list(cs, args):
     """Lists all volumes."""
     # NOTE(thingee): Backwards-compatibility with v1 args
@@ -137,9 +142,10 @@ def do_list(cs, args):
         'migration_status': args.migration_status,
         'metadata': shell_utils.extract_metadata(args)
         if args.metadata else None,
-        'glance_metadata': shell.utils.extract_metadata(args,
+        'glance_metadata': shell_utils.extract_metadata(args,
                                                         type='image_metadata')
-        if args.image_metadata else None,
+        if hasattr(args, 'image_metadata') and args.image_metadata else None,
+        'group_id': getattr(args, 'group_id', None),
     }
 
     # If unavailable/non-existent fields are specified, these fields will
@@ -167,7 +173,11 @@ def do_list(cs, args):
         setattr(vol, 'attached_to', ','.join(map(str, servers)))
 
     if field_titles:
-        key_list = ['ID'] + field_titles
+        # Remove duplicate fields
+        key_list = ['ID']
+        unique_titles = [k for k in collections.OrderedDict.fromkeys(
+            [x.title().strip() for x in field_titles]) if k != 'Id']
+        key_list.extend(unique_titles)
     else:
         key_list = ['ID', 'Status', 'Name', 'Size', 'Volume Type',
                     'Bootable', 'Attached to']
@@ -184,7 +194,6 @@ def do_list(cs, args):
                      sortby_index=sortby_index)
 
 
-@utils.service_type('volumev3')
 @utils.arg('size',
            metavar='<size>',
            nargs='?',
@@ -361,7 +370,6 @@ def do_create(cs, args):
            start_version='3.15',
            help='Metadata key and value pair to set or unset. '
                 'For unset, specify only the key(s): <key key>')
-@utils.service_type('volumev3')
 def do_metadata(cs, args):
     """Sets or deletes volume metadata."""
     volume = utils.find_volume(cs, args.volume)
@@ -375,7 +383,6 @@ def do_metadata(cs, args):
                                    reverse=True))
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 def do_group_type_list(cs, args):
     """Lists available 'group types'. (Admin only will see private types)"""
@@ -383,7 +390,6 @@ def do_group_type_list(cs, args):
     shell_utils.print_group_type_list(gtypes)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 def do_group_type_default(cs, args):
     """List the default group type."""
@@ -391,7 +397,6 @@ def do_group_type_default(cs, args):
     shell_utils.print_group_type_list([gtype])
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 @utils.arg('group_type',
            metavar='<group_type>',
@@ -406,7 +411,6 @@ def do_group_type_show(cs, args):
     utils.print_dict(info, formatters=['group_specs'])
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 @utils.arg('id',
            metavar='<id>',
@@ -428,7 +432,6 @@ def do_group_type_update(cs, args):
     shell_utils.print_group_type_list([gtype])
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 def do_group_specs_list(cs, args):
     """Lists current group types and specs."""
@@ -436,7 +439,6 @@ def do_group_specs_list(cs, args):
     utils.print_list(gtypes, ['ID', 'Name', 'group_specs'])
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 @utils.arg('name',
            metavar='<name>',
@@ -455,7 +457,6 @@ def do_group_type_create(cs, args):
     shell_utils.print_group_type_list([gtype])
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 @utils.arg('group_type',
            metavar='<group_type>', nargs='+',
@@ -477,7 +478,6 @@ def do_group_type_delete(cs, args):
                                       "specified types.")
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.11')
 @utils.arg('gtype',
            metavar='<gtype>',
@@ -543,7 +543,6 @@ def do_group_type_key(cs, args):
            metavar='<per_volume_gigabytes>',
            type=int, default=None,
            help='Set max volume size limit. Default=None.')
-@utils.service_type('volumev3')
 def do_quota_update(cs, args):
     """Updates quotas for a tenant."""
 
@@ -592,7 +591,6 @@ def do_quota_update(cs, args):
            help='Prevents image from being deleted. Default=False.',
            default=False,
            start_version='3.1')
-@utils.service_type('volumev3')
 def do_upload_to_image(cs, args):
     """Uploads volume to Image Service as an image."""
     volume = utils.find_volume(cs, args.volume)
@@ -612,7 +610,6 @@ def do_upload_to_image(cs, args):
                                    args.disk_format))
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.9')
 @utils.arg('backup', metavar='<backup>',
            help='Name or ID of backup to rename.')
@@ -637,7 +634,6 @@ def do_backup_update(cs, args):
     shell_utils.find_backup(cs, args.backup).update(**kwargs)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.7')
 @utils.arg('--name', metavar='<name>', default=None,
            help='Filter by cluster name, without backend will list all '
@@ -672,7 +668,6 @@ def do_cluster_list(cs, args):
     utils.print_list(clusters, columns)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.7')
 @utils.arg('binary', metavar='<binary>', nargs='?', default='cinder-volume',
            help='Binary to filter by.  Default: cinder-volume.')
@@ -684,7 +679,6 @@ def do_cluster_show(cs, args):
     utils.print_dict(cluster.to_dict())
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.7')
 @utils.arg('binary', metavar='<binary>', nargs='?', default='cinder-volume',
            help='Binary to filter by.  Default: cinder-volume.')
@@ -696,7 +690,6 @@ def do_cluster_enable(cs, args):
     utils.print_dict(cluster.to_dict())
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.7')
 @utils.arg('binary', metavar='<binary>', nargs='?', default='cinder-volume',
            help='Binary to filter by.  Default: cinder-volume.')
@@ -711,7 +704,6 @@ def do_cluster_disable(cs, args):
     utils.print_dict(cluster.to_dict())
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.8')
 @utils.arg('host',
            metavar='<host>',
@@ -741,7 +733,7 @@ def do_cluster_disable(cs, args):
            help=(('Comma-separated list of sort keys and directions in the '
                   'form of <key>[:<asc|desc>]. '
                   'Valid keys: %s. '
-                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+                  'Default=None.') % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
 def do_manageable_list(cs, args):
     """Lists all manageable volumes."""
     detailed = strutils.bool_from_string(args.detailed)
@@ -754,7 +746,6 @@ def do_manageable_list(cs, args):
     utils.print_list(volumes, columns, sortby_index=None)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.13')
 @utils.arg('--all-tenants',
            dest='all_tenants',
@@ -762,17 +753,32 @@ def do_manageable_list(cs, args):
            nargs='?',
            type=int,
            const=1,
-           default=0,
+           default=utils.env('ALL_TENANTS', default=0),
            help='Shows details for all tenants. Admin only.')
 def do_group_list(cs, args):
     """Lists all groups."""
-    groups = cs.groups.list()
+    search_opts = {'all_tenants': args.all_tenants}
+
+    groups = cs.groups.list(search_opts=search_opts)
 
     columns = ['ID', 'Status', 'Name']
     utils.print_list(groups, columns)
 
 
-@utils.service_type('volumev3')
+@api_versions.wraps('3.13')
+@utils.arg('group',
+           metavar='<group>',
+           help='Name or ID of a group.')
+def do_group_show(cs, args):
+    """Shows details of a group."""
+    info = dict()
+    group = shell_utils.find_group(cs, args.group)
+    info.update(group._info)
+
+    info.pop('links', None)
+    utils.print_dict(info)
+
+
 @api_versions.wraps('3.13')
 @utils.arg('grouptype',
            metavar='<group-type>',
@@ -809,7 +815,6 @@ def do_group_create(cs, args):
     utils.print_dict(info)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.14')
 @utils.arg('--group-snapshot',
            metavar='<group-snapshot>',
@@ -850,7 +855,6 @@ def do_group_create_from_src(cs, args):
     utils.print_dict(info)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.13')
 @utils.arg('group',
            metavar='<group>', nargs='+',
@@ -880,7 +884,6 @@ def do_group_delete(cs, args):
                                       "groups.")
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.13')
 @utils.arg('group',
            metavar='<group>',
@@ -923,7 +926,6 @@ def do_group_update(cs, args):
     shell_utils.find_group(cs, args.group).update(**kwargs)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.14')
 @utils.arg('--all-tenants',
            dest='all_tenants',
@@ -958,7 +960,6 @@ def do_group_snapshot_list(cs, args):
     utils.print_list(group_snapshots, columns)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.14')
 @utils.arg('group_snapshot',
            metavar='<group_snapshot>',
@@ -973,7 +974,6 @@ def do_group_snapshot_show(cs, args):
     utils.print_dict(info)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.14')
 @utils.arg('group',
            metavar='<group>',
@@ -1002,7 +1002,6 @@ def do_group_snapshot_create(cs, args):
     utils.print_dict(info)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.14')
 @utils.arg('group_snapshot',
            metavar='<group_snapshot>', nargs='+',
@@ -1022,7 +1021,7 @@ def do_group_snapshot_delete(cs, args):
                                       "group snapshots.")
 
 
-@api_versions.wraps('3.7')
+@api_versions.wraps('3.0')
 @utils.arg('--host', metavar='<hostname>', default=None,
            help='Host name. Default=None.')
 @utils.arg('--binary', metavar='<binary>', default=None,
@@ -1032,13 +1031,17 @@ def do_group_snapshot_delete(cs, args):
            const=True,
            nargs='?',
            default=False,
+           start_version='3.7',
            help='Enables or disables display of '
                 'Replication info for c-vol services. Default=False.')
-@utils.service_type('volumev3')
 def do_service_list(cs, args):
     """Lists all services. Filter by host and service binary."""
-    replication = strutils.bool_from_string(args.withreplication,
-                                            strict=True)
+    if hasattr(args, 'withreplication'):
+        replication = strutils.bool_from_string(args.withreplication,
+                                                strict=True)
+    else:
+        replication = False
+
     result = cs.services.list(host=args.host, binary=args.binary)
     columns = ["Binary", "Host", "Zone", "Status", "State", "Updated_at"]
     if cs.api_version.matches('3.7'):
@@ -1052,7 +1055,6 @@ def do_service_list(cs, args):
     utils.print_list(result, columns)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps('3.8')
 @utils.arg('host',
            metavar='<host>',
@@ -1082,7 +1084,7 @@ def do_service_list(cs, args):
            help=(('Comma-separated list of sort keys and directions in the '
                   'form of <key>[:<asc|desc>]. '
                   'Valid keys: %s. '
-                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+                  'Default=None.') % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
 def do_snapshot_manageable_list(cs, args):
     """Lists all manageable snapshots."""
     detailed = strutils.bool_from_string(args.detailed)
@@ -1098,7 +1100,6 @@ def do_snapshot_manageable_list(cs, args):
     utils.print_list(snapshots, columns, sortby_index=None)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps("3.0")
 def do_api_version(cs, args):
     """Display the server API version information."""
@@ -1106,8 +1107,6 @@ def do_api_version(cs, args):
     response = cs.services.server_api_version()
     utils.print_list(response, columns)
 
-
-@utils.service_type('volumev3')
 @api_versions.wraps("3.3")
 @utils.arg('--marker',
            metavar='<marker>',
@@ -1179,7 +1178,6 @@ def do_message_list(cs, args):
     utils.print_list(messages, columns, sortby_index=sortby_index)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps("3.3")
 @utils.arg('message',
            metavar='<message>',
@@ -1193,7 +1191,6 @@ def do_message_show(cs, args):
     utils.print_dict(info)
 
 
-@utils.service_type('volumev3')
 @api_versions.wraps("3.3")
 @utils.arg('message',
            metavar='<message>', nargs='+',
@@ -1210,3 +1207,318 @@ def do_message_delete(cs, args):
     if failure_count == len(args.message):
         raise exceptions.CommandError("Unable to delete any of the specified "
                                       "messages.")
+
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
+@utils.arg('--all_tenants',
+           nargs='?',
+           type=int,
+           const=1,
+           help=argparse.SUPPRESS)
+@utils.arg('--name',
+           metavar='<name>',
+           default=None,
+           help='Filters results by a name. Default=None.')
+@utils.arg('--display-name',
+           help=argparse.SUPPRESS)
+@utils.arg('--display_name',
+           help=argparse.SUPPRESS)
+@utils.arg('--status',
+           metavar='<status>',
+           default=None,
+           help='Filters results by a status. Default=None.')
+@utils.arg('--volume-id',
+           metavar='<volume-id>',
+           default=None,
+           help='Filters results by a volume ID. Default=None.')
+@utils.arg('--volume_id',
+           help=argparse.SUPPRESS)
+@utils.arg('--marker',
+           metavar='<marker>',
+           default=None,
+           help='Begin returning snapshots that appear later in the snapshot '
+                'list than that represented by this id. '
+                'Default=None.')
+@utils.arg('--limit',
+           metavar='<limit>',
+           default=None,
+           help='Maximum number of snapshots to return. Default=None.')
+@utils.arg('--sort',
+           metavar='<key>[:<direction>]',
+           default=None,
+           help=(('Comma-separated list of sort keys and directions in the '
+                  'form of <key>[:<asc|desc>]. '
+                  'Valid keys: %s. '
+                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+@utils.arg('--tenant',
+           type=str,
+           dest='tenant',
+           nargs='?',
+           metavar='<tenant>',
+           help='Display information from single tenant (Admin only).')
+@utils.arg('--metadata',
+           nargs='*',
+           metavar='<key=value>',
+           default=None,
+           start_version='3.22',
+           help='Filters results by a metadata key and value pair. Require '
+                'volume api version >=3.22. Default=None.')
+def do_snapshot_list(cs, args):
+    """Lists all snapshots."""
+    all_tenants = (1 if args.tenant else
+                   int(os.environ.get("ALL_TENANTS", args.all_tenants)))
+
+    if args.display_name is not None:
+        args.name = args.display_name
+
+    metadata = None
+    try:
+        if args.metadata:
+            metadata = shell_utils.extract_metadata(args)
+    except AttributeError:
+        pass
+
+    search_opts = {
+        'all_tenants': all_tenants,
+        'name': args.name,
+        'status': args.status,
+        'volume_id': args.volume_id,
+        'project_id': args.tenant,
+        'metadata': metadata
+    }
+
+    snapshots = cs.volume_snapshots.list(search_opts=search_opts,
+                                         marker=args.marker,
+                                         limit=args.limit,
+                                         sort=args.sort)
+    shell_utils.translate_volume_snapshot_keys(snapshots)
+    sortby_index = None if args.sort else 0
+    utils.print_list(snapshots,
+                     ['ID', 'Volume ID', 'Status', 'Name', 'Size'],
+                     sortby_index=sortby_index)
+
+@api_versions.wraps('3.27')
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
+@utils.arg('--volume-id',
+           metavar='<volume-id>',
+           default=None,
+           help='Filters results by a volume ID. Default=None.')
+@utils.arg('--status',
+           metavar='<status>',
+           default=None,
+           help='Filters results by a status. Default=None.')
+@utils.arg('--marker',
+           metavar='<marker>',
+           default=None,
+           help='Begin returning attachments that appear later in '
+                'attachment list than that represented by this id. '
+                'Default=None.')
+@utils.arg('--limit',
+           metavar='<limit>',
+           default=None,
+           help='Maximum number of attachemnts to return. Default=None.')
+@utils.arg('--sort',
+           metavar='<key>[:<direction>]',
+           default=None,
+           help=(('Comma-separated list of sort keys and directions in the '
+                  'form of <key>[:<asc|desc>]. '
+                  'Valid keys: %s. '
+                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+@utils.arg('--tenant',
+           type=str,
+           dest='tenant',
+           nargs='?',
+           metavar='<tenant>',
+           help='Display information from single tenant (Admin only).')
+def do_attachment_list(cs, args):
+    """Lists all attachments."""
+    search_opts = {
+        'all_tenants': args.all_tenants,
+        'status': args.status,
+        'volume_id': args.volume_id,
+    }
+
+    attachments = cs.attachments.list(search_opts=search_opts,
+                                      marker=args.marker,
+                                      limit=args.limit,
+                                      sort=args.sort)
+    columns = ['ID', 'Volume ID', 'Status', 'Instance']
+    if args.sort:
+        sortby_index = None
+    else:
+        sortby_index = 0
+    utils.print_list(attachments, columns, sortby_index=sortby_index)
+
+
+@api_versions.wraps('3.27')
+@utils.arg('attachment',
+           metavar='<attachment>',
+           help='ID of attachment.')
+def do_attachment_show(cs, args):
+    """Show detailed information for attachment."""
+    attachment = cs.attachments.show(args.attachment)
+    attachment_dict = attachment.to_dict()
+    connection_dict = attachment_dict.pop('connection_info', {})
+    utils.print_dict(attachment_dict)
+
+    # TODO(jdg): Need to add checks here like admin/policy for displaying the
+    # connection_info, this is still experimental so we'll leave it enabled for
+    # now
+    if connection_dict:
+        utils.print_dict(connection_dict)
+
+
+@api_versions.wraps('3.27')
+@utils.arg('volume',
+           metavar='<volume>',
+           help='Name or ID of volume or volumes to attach.')
+@utils.arg('--instance',
+           metavar='<instance>',
+           default=None,
+           help='UUID of Instance attaching to. Default=None.')
+@utils.arg('--connect',
+           metavar='<connect>',
+           default=False,
+           help='Make an active connection using provided connector info '
+                '(True or False).')
+@utils.arg('--initiator',
+           metavar='<initiator>',
+           default=None,
+           help='iqn of the initiator attaching to.  Default=None.')
+@utils.arg('--ip',
+           metavar='<ip>',
+           default=None,
+           help='ip of the system attaching to.  Default=None.')
+@utils.arg('--host',
+           metavar='<host>',
+           default=None,
+           help='Name of the host attaching to. Default=None.')
+@utils.arg('--platform',
+           metavar='<platform>',
+           default='x86_64',
+           help='Platform type. Default=x86_64.')
+@utils.arg('--ostype',
+           metavar='<ostype>',
+           default='linux2',
+           help='OS type. Default=linux2.')
+@utils.arg('--multipath',
+           metavar='<multipath>',
+           default=False,
+           help='OS type. Default=False.')
+@utils.arg('--mountpoint',
+           metavar='<mountpoint>',
+           default=None,
+           help='Mountpoint volume will be attached at. Default=None.')
+def do_attachment_create(cs, args):
+    """Create an attachment for a cinder volume."""
+
+    connector = {}
+    if strutils.bool_from_string(args.connect, strict=True):
+        # FIXME(jdg): Add in all the options when they're finalized
+        connector = {'initiator': args.initiator,
+                     'ip': args.ip,
+                     'platform': args.platform,
+                     'host': args.host,
+                     'os_type': args.ostype,
+                     'multipath': args.multipath}
+    attachment = cs.attachments.create(args.volume,
+                                       connector,
+                                       args.instance)
+    connector_dict = attachment.pop('connection_info', None)
+    utils.print_dict(attachment)
+    if connector_dict:
+        utils.print_dict(connector_dict)
+
+
+@api_versions.wraps('3.27')
+@utils.arg('attachment',
+           metavar='<attachment>',
+           help='ID of attachment.')
+@utils.arg('--initiator',
+           metavar='<initiator>',
+           default=None,
+           help='iqn of the initiator attaching to.  Default=None.')
+@utils.arg('--ip',
+           metavar='<ip>',
+           default=None,
+           help='ip of the system attaching to.  Default=None.')
+@utils.arg('--host',
+           metavar='<host>',
+           default=None,
+           help='Name of the host attaching to. Default=None.')
+@utils.arg('--platform',
+           metavar='<platform>',
+           default='x86_64',
+           help='Platform type. Default=x86_64.')
+@utils.arg('--ostype',
+           metavar='<ostype>',
+           default='linux2',
+           help='OS type. Default=linux2.')
+@utils.arg('--multipath',
+           metavar='<multipath>',
+           default=False,
+           help='OS type. Default=False.')
+@utils.arg('--mountpoint',
+           metavar='<mountpoint>',
+           default=None,
+           help='Mountpoint volume will be attached at. Default=None.')
+def do_attachment_update(cs, args):
+    """Update an attachment for a cinder volume.
+    This call is designed to be more of an attachment completion than anything
+    else.  It expects the value of a connector object to notify the driver that
+    the volume is going to be connected and where it's being connected to.
+    """
+    connector = {'initiator': args.initiator,
+                 'ip': args.ip,
+                 'platform': args.platform,
+                 'host': args.host,
+                 'os_type': args.ostype,
+                 'multipath': args.multipath}
+    attachment = cs.attachments.update(args.attachment,
+                                       connector)
+    attachment_dict = attachment.to_dict()
+    connector_dict = attachment_dict.pop('connection_info', None)
+    utils.print_dict(attachment_dict)
+    if connector_dict:
+        utils.print_dict(connector_dict)
+
+
+@api_versions.wraps('3.27')
+@utils.arg('attachment',
+           metavar='<attachment>', nargs='+',
+           help='ID of attachment or attachments to delete.')
+def do_attachment_delete(cs, args):
+    """Delete an attachment for a cinder volume."""
+    for attachment in args.attachment:
+        cs.attachments.delete(attachment)
+
+@api_versions.wraps('3.0')
+def do_version_list(cs, args):
+    """List all API versions."""
+    result = cs.services.server_api_version()
+    if 'min_version' in dir(result[0]):
+        columns = ["Id", "Status", "Updated", "Min Version", "Version"]
+    else:
+        columns = ["Id", "Status", "Updated"]
+
+    print(("Client supported API versions:"))
+    print(("Minimum version %(v)s") %
+          {'v': api_versions.MIN_VERSION})
+    print(("Maximum version %(v)s") %
+          {'v': api_versions.MAX_VERSION})
+
+    print(("\nServer supported API versions:"))
+    utils.print_list(result, columns)
